!---------------------------------------------------------------------------------------
! FLEXINVERT: m1qn3_interface
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> m1qn3_interface
!!
!! Purpose:
!!   Interface with the Quasi-Newton M1QN3 algorithm
!!
!! Interface:
!!     uses Reverse Protocol 
!!     calls simulation routine (to calculate the gradient and cost) and 
!!     M1QN3 iteratively until reach required precision or max no. iterations
!!     n                 ! number of variables
!!     x                 ! state vector, on first entry prior state vector 
!!     f                 ! value of the cost function
!!     g                 ! gradient of the cost function
!
!---------------------------------------------------------------------------------------

subroutine m1qn3_interface(iter, grad, files, config, fluxes, obs, states, covar, cost)

  use mod_var
  use mod_settings
  use mod_fluxes
  use mod_obs
  use mod_states
  use mod_covar
  use mod_transform
  use mod_m1qn3

  implicit none

  type (files_t),                  intent (in)     :: files
  type (config_t),                 intent (in)     :: config
  type (fluxes_t),                 intent (in)     :: fluxes
  type (obs_t),                    intent (in)     :: obs
  type (states_t),                 intent (in out) :: states
  type (covar_t),                  intent (in out) :: covar
  real(kind=8), dimension(nvar,1), intent (in out) :: grad
  real,                            intent (in out) :: cost
  integer,                         intent (in out) :: iter

  ! local variables that need to be initialized
  character(len=3)                          :: normtype
  integer                                   :: i,n,imp,io,imode(3),omode,nits,nsim,iz(5),ndz,izs(1)
  integer                                   :: indic,reverse, ierr
  real                                      :: rzs(1)
  real(kind=8)                              :: f,dxmin,df1,epsrel,dzs(1)  ! reverse protocol dzs is a dummy variable
  real(kind=8), dimension(nvar)             :: x, g
  real(kind=8), dimension(:), allocatable   :: dz

  ! local variables for interface with simulate.f90
  real                                      :: cost_o, cost_p
  real(kind=8), dimension(nvar,1)           :: phi, chi_ad, px
  real, dimension(nvar)                     :: grad_o
  character(len=2)                          :: aiter

  ! variables defined in mod_m1qn3 but need to be saved 
  logical                                   :: inmemo,sscale
  integer                                   :: ntravu,id,igg,idiag,iaux,ialpha,iybar,isbar,m,mmemo,reentry,isim
  real(kind=8)                              :: gnorm,gnorms,ps,dk,dk1
  logical                                   :: cold,warm,skip_update
  integer                                   :: itmax,moderl,jcour
  real(kind=8)                              :: d1,t,tmin,tmax,eps1,ff,preco,precos,ys,den,ps2,hp0

  ! variables defined in mlis3 but need to be saved
  logical                                   :: t_increased
  integer                                   :: indica,indicd
  real(kind=8)                              :: tesf,tesd,tg,fg,fpg,td,ta,fa,fpa,d2,fn,fp,ffn,fd
  real(kind=8)                              :: fpd,z,test,barmin,barmul,barmax,barr,gauche,droite,taa


  ! initialize local variables
  ! --------------------------

  n = nvar
  ! initial guess at vector that minimizes the cost in chi space
  x(:) = 0d0
  ! initial gradient in chi space
  g(:) = grad(:,1)
  ! initial cost
  f = cost
  ! save initial cost in case of a restart
  open(100,file=trim(files%path_output)//'cost.txt',status='unknown',access='append',action='write',iostat=ierr)
  write(100,fmt='(E11.4)') cost
  close(100)

  ! initialize M1QN3 settings
  ! -------------------------

  isim=1
  ndz=6*n+1
  indic=4
  dxmin=1.e-20
  df1=f
  epsrel=1.e-10
  nits=maxiter
  nsim=100
  normtype='dfn'
  io=6
  imp=5
  imode(1)=0
  imode(2)=0
  imode(3)=0
  reverse=1
  reentry=0
  ! workspace for M1QN3
  allocate( dz(ndz) )
  dz(:)=0d0

  ! interface to M1QN3
  ! ------------------

  iterate: do

    call m1qn3(n,x,f,g,dxmin,df1,&
                epsrel,normtype,imp,io,imode,omode,nits,nsim,iz, &
                dz,ndz,reverse,indic,isim,dzs,&
                inmemo,sscale,ntravu,id,igg,idiag,iaux,ialpha,iybar,isbar,m,&
                mmemo,reentry,gnorm,gnorms,ps,dk,dk1,&
                cold,warm,skip_update,&
                itmax,moderl,jcour,&
                d1,t,tmin,tmax,eps1,ff,preco,precos,ys,den,ps2,hp0,&
                t_increased,indica,indicd,&
                tesf,tesd,tg,fg,fpg,td,ta,fa,fpa,d2,fn,fp,ffn,fd,&
                fpd,z,test,barmin,barmul,barmax,barr,gauche,droite,taa)
          
    iter = iter + 1
    if (reverse.lt.0) exit iterate
    if(iter.gt.maxiter) exit iterate
 
    ! calculate new gradient
    ! ----------------------

    if ( (config%restart.eq.1).and.(iter.le.lastiter) ) then
      ! use precalculated gradient
      write(aiter,fmt='(I2.2)') iter
      open(100,file=trim(files%path_output)//'grad_'//aiter//'.txt',action='read',status='old',iostat=ierr)
      if ( ierr.ne.0 ) then
        write(logid,*) 'ERROR m1qn3_interface: cannot open grad_'//aiter//'.txt'
        stop
      endif
      do i = 1, nvar
        read(100,*) grad_o(i)
      end do
      close(100)
      ! use precalculated cost
      open(100,file=trim(files%path_output)//'cost.txt',action='read',status='old',iostat=ierr)
      if ( ierr.ne.0 ) then
        write(logid,*) 'ERROR m1qn3_interface: cannot open cost.txt'
        stop
      endif
      do i = 1, (iter+1)
        read(100,*) cost
      end do
      close(100)      
      write(logid,*) 'Cost read for iteration '//aiter//': ',cost
      f = cost 
      ! transform x to physical space (only needed if lastiter = maxiter)
      px = chi2phi(config, states, covar, x)
      states%px = real(px(:,1),kind=4)
    else
      ! transform x to physical space
      px = chi2phi(config, states, covar, x)
      states%px = real(px(:,1),kind=4)
      ! run model
      cost_o = 0.
      grad_o(:) = 0.
      call simulate(iter, files, config, fluxes, obs, states, grad_o, cost_o)
      ! calculate cost
      ! Jp = x^Tx where x = state vector in chi space
      cost_p = dot_product(x,x) 
      ! J = 0.5*(Jp + Jo) 
      cost = 0.5*(cost_p + cost_o)
      f = cost
      write(aiter,'(I2.2)') iter
      write(logid,*) 'Cost iteration '//aiter//': ',cost
      ! save cost in case of a restart
      open(100,file=trim(files%path_output)//'cost.txt',status='unknown',access='append',action='write',iostat=ierr)
      write(100,fmt='(E11.4)') cost
      close(100)
    endif

    ! transform gradient to chi space
    phi(:,1) = dble(grad_o)
    chi_ad = phi2chi_ad(config, covar, phi )

    ! new gradient J' = J'p + J'o
    g(:) = x(:) + chi_ad(:,1)

  end do iterate


end subroutine m1qn3_interface



