!---------------------------------------------------------------------------------------
! FLEXINVERT: mod_var
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> mod_var
!!
!---------------------------------------------------------------------------------------


module mod_var

  implicit none

  ! fixed parameters 
  ! -----------------

  integer, parameter :: max_path_len=200         ! max character length of paths and files
  integer, parameter :: max_name_len=50          ! max character length of variable names
  integer, parameter :: recname_len=7            ! length of receptor names
  real, parameter    :: numscale=1.e12           ! numeric scaling factor
  integer, parameter :: maxpoint=1000            ! max number of releases per receptor per month
  integer, parameter :: maxspec=10               ! max number of species in a flexpart run
  integer, parameter :: maxlev=50                ! max number of vertical levels in flexpart output
  real, parameter    :: trunc=1.e-15             ! truncation threshold for eigenvalues of error covariance matrix
  real, parameter    :: reqrd = 1.e-15           ! required reduction in gradient norm for congrad

  ! flexpart variables 
  ! ------------------

  ! generic
  integer            :: ageclass                 ! length of backtrajectories (s)
  integer            :: trajdays                 ! length of backtrajectories (days)
  integer            :: ndgrid                   ! number of footprints per day
  integer            :: maxngrid                 ! max number of footprints (ndgrid*trajdays+2)
  integer            :: nxshift                  ! longitudinal offset in grid cells relative to 180W for global output
  integer            :: nnxshift                 ! longitudinal offset in grid cells relative to edge of nest for nested output
  integer            :: nzgrid                   ! number of vertical layers in grid_init files
  integer            :: nzlev                    ! number of vertical layers in grid_time files (nzlev = 1)
  real, dimension(maxlev) :: outheight           ! height of model layers for grid_init files (m)

  ! global domain
  integer            :: nxgrid                   ! number of longitudinal grid cells
  integer            :: nygrid                   ! number of latitudinal grid cells
  real               :: llx                      ! lower right longitude of flexpart grid
  real               :: lly                      ! lower right latitude of flexpart grid
  real               :: dx                       ! longitudinal resolution of flexpart grid
  real               :: dy                       ! latitudinal resolution of flexpart grid
  real, dimension(:), allocatable :: gbl_lon     ! longitudes of flexpart domain
  real, dimension(:), allocatable :: gbl_lat     ! latitudes of flexpart domain

  ! nested domain
  integer            :: nnxgrid                  ! number of longitudinal grid cells
  integer            :: nnygrid                  ! number of latitudinal grid cells
  real               :: nllx                     ! lower right longitude of flexpart grid
  real               :: nlly                     ! lower right latitude of flexpart grid
  real               :: ndx                      ! longitudinal resolution of flexpart grid
  real               :: ndy                      ! latitudinal resolution of flexpart grid
  real, dimension(:), allocatable :: nest_lon    ! longitudes of flexpart domain
  real, dimension(:), allocatable :: nest_lat    ! latitudes of flexpart domain

  ! physical and other constants
  !-----------------------------

  real, parameter    :: pi=3.14159265            ! pi
  real, parameter    :: rearth=6.371e6           ! radius of earth (m)
  real, parameter    :: mmair=28.97              ! standard molecular mass of air
  real, parameter    :: gasc=8.314               ! ideal gas constant (J/K/mol)
  real, parameter    :: psurf=101325             ! surface pressure (Pa)
  real, parameter    :: gc=9.81                  ! gravitational acceleration (m/s2)
  real, parameter    :: ts=288.15                ! surface temperature (K)
  real, parameter    :: lrate=0.0065             ! lapse rate (K/m)
  real, parameter    :: mmol=0.02894             ! (kg/mol)

  ! time variables
  ! --------------

  real(kind=8)       :: juldatei                 ! julian start date
  real(kind=8)       :: juldatef                 ! julian end date
  integer            :: nday                     ! number of days in inversion interval
  integer            :: nd_nee                   ! number of NEE values per day (in prior NEE file)
  integer            :: ntstate                  ! number of time steps in state vector
  integer            :: ntflx                    ! number of time steps in non-NEE fluxes
  real               :: statres                  ! time resolution of the state vector (for CO2 1 day) (days) 
  real               :: statres_hr               ! subdaily time resolution of the state vector (for CO2 e.g. 6, 12 or 24 h) 
  real               :: flxtres                  ! time resolution of fluxes (for GHG same as statres) (days) 

  ! grid variables
  ! --------------

  real               :: rllx                      ! lower right longitude of inversion domain
  real               :: rlly                      ! lower right latitude of inversion domain
  real               :: rurx                      ! upper right longitude of inversion domain
  real               :: rury                      ! upper right latitude of inversion domain
  real               :: rdx                       ! longitudinal resolution in inversion domain
  real               :: rdy                       ! latitudinal resolution in inversion domain
  integer            :: nxregrid                  ! number of longitudinal grid cells in inversion domain 
  integer            :: nyregrid                  ! number of latitudinal grid cells in inversion domain
  real, dimension(:), allocatable   :: reg_lon    ! longitudes of inversion domain
  real, dimension(:), allocatable   :: reg_lat    ! latitudes of inversion domain
  real, dimension(:,:), allocatable :: lsm        ! global land-sea mask  
 
  ! region variables
  ! ----------------

  integer                              :: nbox    ! number of boxes in inversion domain
  integer                              :: nocn    ! number of ocean boxes in inversion domain (not optimized)
  integer, dimension(:,:), allocatable :: nbox_xy ! projection operator for boxes
  integer, dimension(:,:), allocatable :: lsm_box ! projection operator for land-sea mask
  real, dimension(:), allocatable      :: area_box! area of boxes
  real, dimension(:), allocatable      :: area_ocn! area of ocean boxes (not optimized)
  real, dimension(:), allocatable      :: hloc    ! local time shift wrt UTC for boxes (h)
  real, dimension(:,:), allocatable    :: hmat    ! regional transport operator for 'analytic' method

  ! observation variables
  ! ---------------------

  integer                              :: nrec    ! number of receptor sites or campaigns
  integer                              :: nobs    ! total number of observations
  character(recname_len), dimension(:), allocatable :: recname ! names of receptors

  ! state variables
  ! ---------------

  integer            :: nvar                      ! total number of state variables 
  integer            :: npvar                     ! number of state variables related to fluxes
  integer            :: ndt                       ! number of state vector time steps per day (for CO2)
  integer            :: ndvar                     ! number of variables per flux time step (for CO2 2*nbox else nbox) 
  integer, parameter :: ncini=4                   ! number of initial mixing ratio variables to optimize
  real, parameter, dimension(4) :: cini_lat=(/-30,0,30,90/) ! vector of northern edges of latitude bands for initial mixing ratio optimization 
  integer            :: ntcini                    ! number of initial mixing ratio time steps 
  integer, parameter :: cinires=28                ! time resolution for initial mixing ratio scalars (days) 
  integer            :: neig                      ! number of retained eigenvalues 
  integer            :: lastiter                  ! number of last iteration (if restart is true)
  integer            :: maxiter                   ! max number of iterations for congrad and m1qn3

  ! ouput files
  ! -----------
 
  integer, parameter :: logid = 500


end module mod_var

